-- functions
local function NLUIGVKTSortKeyList(NLUIGVKTWCTList)-- sorts the list of key levels from highest to lowest. NLUIGVKTWCTList[1] being the highest.
    local sortedKeyList = NLUIGVKTWCTList
    local i = 2;
    while i <= table.getn(sortedKeyList) do
        local j = i - 1;
        local num1 = sortedKeyList[i];
        while j > 0 and sortedKeyList[j] < num1 do
            sortedKeyList[j + 1] = sortedKeyList[j]
            j = j - 1;
        end
        sortedKeyList[j + 1] = num1
        i = i + 1;
    end
    return sortedKeyList
end

local function NLUIGVKTGetWCTList()-- returns a table containing a sorted list of key levels from highest to lowest.

    --uses C_MythicPlus.GetRunHistory(false, true) to get the mythic runs, and then puts them into a new table
    --containing the key levels from this week, then calls a sorting algorithm to sort it from highest to lowest.
    local NLUIGVKTWCTList = {}
    C_MythicPlus.RequestMapInfo()
    local runs = C_MythicPlus.GetRunHistory(false, true)
    for i = 1, #runs do
        if(runs[i].thisWeek == true and runs[i].runScore > 0) then
            table.insert(NLUIGVKTWCTList, runs[i].level)
        end
    end
    return NLUIGVKTSortKeyList(NLUIGVKTWCTList)
end

local function NLUIGVKTGetRewardString(NLUIGVKTWCTList)-- takes a table of the top ten keys, and returns a string that shows the three reward options based off the keys completed so far.
    
    --checks the index 1, 4, and 10 to see if the first, second, and third great vault rewards criteria have been met, and finds their current item level
    --rewards for each.
    --[[
    if C_MythicPlus.IsWeeklyRewardAvailable() == true then
        return 
        maybe try C_MythicPlus.RequestRewards() to see if when this isnt nil then maybe theres something available in the great vault?
        
    end]]
    local rewardText = "\n\n";
    if NLUIGVKTWCTList[1] ~= nil then
        local reward = C_MythicPlus.GetRewardLevelFromKeystoneLevel(NLUIGVKTWCTList[1]);
        rewardText = rewardText.."Reward option 1: "..reward.." ilvl"
        if NLUIGVKTWCTList[4] ~= nil then
            reward = C_MythicPlus.GetRewardLevelFromKeystoneLevel(NLUIGVKTWCTList[4]);
            rewardText = rewardText.."\nReward option 2: "..reward.." ilvl"
            if NLUIGVKTWCTList[8] ~= nil then
                reward = C_MythicPlus.GetRewardLevelFromKeystoneLevel(NLUIGVKTWCTList[8]);
                rewardText = rewardText.."\nReward option 3: "..reward.." ilvl"
            end
        end
    end
    return rewardText;
end

local function NLUIGVKTTopTenToString(NLUIGVKTWCTList)-- takes a table of key levels this week, and returns the top 10 keys achieved in the current week as a string.
    
    --takes a table, and returns its top 10 values from index 1 to index 10
    --colors the 1st, 4th, and 10th green.
    local text = "\nYour top 8 keys this week\n\n";
    local i = 1;
    while i < 9 do
        if NLUIGVKTWCTList[i] ~= nil then
            if i == 1 or i == 4 or i == 8 then
                text = text.."|cff66ff66"..NLUIGVKTWCTList[i].."|r  ";
            else
                text = text..NLUIGVKTWCTList[i].."  ";
            end
            i = i + 1;
        else
            text = text.."0  ";
            i = i + 1;
        end
    end
    return text
end

local function NLUIGVKTGetSpecByClassID_SpecID()-- returns the file path for a given class, and spec.
    --"Interface\\TALENTFRAME\\textures\\bg-deathknight-frost.blp"
    local specID = GetLootSpecialization()
    if specID == 0 then
        specID = GetSpecializationInfo(GetSpecialization())
    end
    local _, name, _, _, _, class = GetSpecializationInfoByID(specID)
    if class == "WARRIOR" then
        if name == "Arms" then
            return "Interface\\TALENTFRAME\\bg-warrior-arms.blp"
        elseif name == "Fury" then
            return "Interface\\TALENTFRAME\\bg-warrior-fury.blp"
        elseif name == "Protection" then
            return "Interface\\TALENTFRAME\\bg-warrior-protection.blp"
        end
    elseif class == "PRIEST" then
        if name == "Discipline" then
            return "Interface\\TALENTFRAME\\bg-priest-discipline.blp"
        elseif name == "Holy" then
            return "Interface\\TALENTFRAME\\bg-priest-holy.blp"
        elseif name == "Shadow" then
            return "Interface\\TALENTFRAME\\bg-priest-shadow.blp"
        end
    elseif class == "SHAMAN" then
        if name == "Elemental" then
            return "Interface\\TALENTFRAME\\bg-shaman-elemental.blp"
        elseif name == "Enhancement" then
            return "Interface\\TALENTFRAME\\bg-shaman-enhancement.blp"
        elseif name == "Restoration" then
            return "Interface\\TALENTFRAME\\bg-shaman-restoration.blp"
        end
    elseif class == "DRUID" then
        if name == "Balance" then
            return "Interface\\TALENTFRAME\\bg-druid-balance.blp"
        elseif name == "Restoration" then
            return "Interface\\TALENTFRAME\\bg-druid-restoration.blp"
        elseif name == "Feral" then
            return "Interface\\TALENTFRAME\\bg-druid-cat.blp"
        elseif name == "Guardian" then
            return "Interface\\TALENTFRAME\\bg-druid-bear.blp"
        end
    elseif class == "DEMONHUNTER" then
        if name == "Havoc" then
            return "Interface\\Addons\\GreatVaultKeyTracker\\textures\\dhHavoc.tga"
        elseif name == "Vengeance" then
            return "Interface\\Addons\\GreatVaultKeyTracker\\textures\\dhVengeance.tga"
        end
    elseif class == "DEATHKNIGHT" then
        if name == "Blood" then
            return "Interface\\TALENTFRAME\\bg-deathknight-blood.blp"
        elseif name == "Frost" then
            return "Interface\\TALENTFRAME\\bg-deathknight-frost.blp"
        elseif name == "Unholy" then
            return "Interface\\TALENTFRAME\\bg-deathknight-unholy.blp"
        end
    elseif class == "PALADIN" then
        if name == "Holy" then
            return "Interface\\TALENTFRAME\\bg-paladin-holy.blp"
        elseif name == "Retribution" then
            return "Interface\\TALENTFRAME\\bg-paladin-retribution.blp"
        elseif name == "Protection" then
            return "Interface\\TALENTFRAME\\bg-paladin-protection.blp"
        end
    elseif class == "MAGE" then
        if name == "Arcane" then
            return "Interface\\TALENTFRAME\\bg-mage-arcane.blp"
        elseif name == "Frost" then
            return "Interface\\TALENTFRAME\\bg-mage-frost.blp"
        elseif name == "Fire" then
            return "Interface\\TALENTFRAME\\bg-mage-fire.blp"
        end
    elseif class == "ROGUE" then
        if name == "Subtlety" then
            return "Interface\\TALENTFRAME\\bg-rogue-subtlety.blp"
        elseif name == "Outlaw" then
            return "Interface\\TALENTFRAME\\bg-rogue-combat.blp"
        elseif name == "Assassination" then
            return "Interface\\TALENTFRAME\\bg-rogue-assassination.blp"
        end
    elseif class == "MONK" then
        if name == "Windwalker" then
            return "Interface\\TALENTFRAME\\bg-monk-battledancer.blp"
        elseif name == "Brewmaster" then
            return "Interface\\TALENTFRAME\\bg-monk-brewmaster.blp"
        elseif name == "Mistweaver" then
            return "Interface\\TALENTFRAME\\bg-monk-mistweaver.blp"
        end
    elseif class == "HUNTER" then
        if name == "Survival" then
            return "Interface\\TALENTFRAME\\bg-hunter-survival.blp"
        elseif name == "Beast Mastery" then
            return "Interface\\TALENTFRAME\\bg-hunter-beastmaster.blp"
        elseif name == "Marksmanship" then
            return "Interface\\TALENTFRAME\\bg-hunter-marksman.blp"
        end
    elseif class == "WARLOCK" then
        if name == "Demonology" then
            return "Interface\\TALENTFRAME\\bg-warlock-demonology.blp"
        elseif name == "Affliction" then
            return "Interface\\TALENTFRAME\\bg-warlock-affliction.blp"
        elseif name == "Destruction" then
            return "Interface\\TALENTFRAME\\bg-warlock-destruction.blp"
        end
    end
    return "Interface\\TALENTFRAME\\bg-warrior-arms.blp"
end

local function NLUIGVKTGetLootSpec()-- returns the name of the current loot spec.
    local specID = GetLootSpecialization()
    if specID == 0 then -- GetSpecializationInfoForClassID(UnitClass("player"))
        specID = GetSpecializationInfo(GetSpecialization())
    end
    local _, spec= GetSpecializationInfoByID(specID)
    return "\n\nCurrent loot spec: "..spec
end

local function NLUIGKTFindKey()
    for bag = 0, NUM_BAG_SLOTS do
        local bagSlots = C_Container.GetContainerNumSlots(bag);
        for bagSlot = 1, bagSlots do
            local itemId = C_Container.GetContainerItemID(bag, bagSlot);
            if itemId == 180653 then
                return {bag, bagSlot};
            end
        end
    end
end

local function NLUIGVKTInsertKey()
    local keyLocation = NLUIGKTFindKey();
    C_Container.PickupContainerItem(keyLocation[1], keyLocation[2])
    C_ChallengeMode.SlotKeystone()
end

local function NLUIGVKTGetWeeklyRewards()-- returns a string containing the seasons weekly reward ilvl for each keystone lvl 2-15
    local rewards = "Weekly Vault\nReward Breakdown\n"
    for keyLevel = 2, 20 do
        local reward = C_MythicPlus.GetRewardLevelForDifficultyLevel(keyLevel);
        if keyLevel % 2 == 1 then
            if keyLevel < 10 then
                rewards = rewards.."|cff66ff66\nKey level: "..keyLevel.."     vault reward: "..reward.."|r"
            else
                rewards = rewards.."|cff66ff66\nKey level: "..keyLevel.."   vault reward: "..reward.."|r"
            end
        else
            if keyLevel < 10 then
                rewards = rewards.."\nKey level: "..keyLevel.."     vault reward: "..reward
            else
                rewards = rewards.."\nKey level: "..keyLevel.."   vault reward: "..reward
            end
        end
    end
    return rewards
end

local function NLUIGVKTGetEndOfDungeonRewards()-- returns a string containing the seasons end of dungeon reward ilvl for each keystone lvl 2-15
    local rewards = "End of Dungeon\nReward Breakdown\n"
    for keyLevel = 2, 20 do
        _, reward = C_MythicPlus.GetRewardLevelForDifficultyLevel(keyLevel)
        if keyLevel % 2 == 1 then
            if keyLevel < 10 then
                rewards = rewards.."|cff66ff66\nKey level: "..keyLevel.."     vault reward: "..reward.."|r"
            else
                rewards = rewards.."|cff66ff66\nKey level: "..keyLevel.."   vault reward: "..reward.."|r"
            end
        else
            if keyLevel < 10 then
                rewards = rewards.."\nKey level: "..keyLevel.."     vault reward: "..reward
            else
                rewards = rewards.."\nKey level: "..keyLevel.."   vault reward: "..reward
            end
        end
    end
    return rewards
end

--local variables
local NLUIGVKTList = {}
local currentKeyLevel
local firstLogin = true;
local frameHeight = 220;
local frameWidth = 300;
local backDropTexture = "";
local weeklyIlvlBreakdown = NLUIGVKTGetWeeklyRewards()
--end of local variables

-- Main parent frame
NLUIGVKT = CreateFrame("Frame","WeeklyChestTracker", UIParent, "BasicFrameTemplateWithInset");
NLUIGVKT:SetSize(frameWidth,frameHeight); -- width, height
NLUIGVKT:SetPoint("TOP",nil,"CENTER",0,0);
NLUIGVKT:EnableMouse(true);
NLUIGVKT:SetMovable(true);
NLUIGVKT:RegisterForDrag("LeftButton");
NLUIGVKT:SetScript("OnDragStart", NLUIGVKT.StartMoving)
NLUIGVKT:SetScript("OnDragStop", NLUIGVKT.StopMovingOrSizing)
NLUIGVKT:SetClampedToScreen(true)
NLUIGVKT:Hide();

--background to the output frame.
--fix if you change specs it should change with your spec as well.
local NLUIBackdrop = NLUIGVKT:CreateTexture()
NLUIBackdrop:SetPoint("CENTER",NLUIGVKT,"CENTER",0,-50)
NLUIBackdrop:SetSize(285,frameHeight+54)
NLUIBackdrop:SetTexture()
-- end of outPutFrame

--register events to handle
NLUIGVKT:RegisterEvent("CHALLENGE_MODE_START");
NLUIGVKT:RegisterEvent("CHALLENGE_MODE_COMPLETED");
NLUIGVKT:RegisterEvent("PLAYER_LOGIN");
NLUIGVKT:RegisterEvent("PLAYER_LOOT_SPEC_UPDATED");
NLUIGVKT:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED");
NLUIGVKT:RegisterEvent("CHALLENGE_MODE_KEYSTONE_RECEPTABLE_OPEN");
NLUIGVKT:RegisterEvent("CHAT_MSG_PARTY");
NLUIGVKT:RegisterEvent("CHAT_MSG_PARTY_LEADER");

--handling events

local function NLUIGVKTEvents(self, event, ...) -- event scanning
    if event == "PLAYER_LOGIN" then -- setting firstLogin to true this will make the first /gvkey call to load in the list from scratch after reloads.
        C_MythicPlus.RequestMapInfo()
        backDropTexture = NLUIGVKTGetSpecByClassID_SpecID()
        currentKeyLevel = NLUIGVKTCurrentKey;
        firstLogin = true;
        if NLUIGVKTBackdropToggle == nil then
            NLUIGVKTBackdropToggle = false
        end
        if NLUIGVKTKeyCommandToggle == nil then
            NLUIGVKTKeyCommandToggle = true
        end
    end
    if event == "PLAYER_LOOT_SPEC_UPDATED" then
        backDropTexture = NLUIGVKTGetSpecByClassID_SpecID()
        NLUIKeyFrame.set();
    end
    if event == "PLAYER_SPECIALIZATION_CHANGED" then
        local unitTarget = ...
        if unitTarget == "player" then
            backDropTexture = NLUIGVKTGetSpecByClassID_SpecID()
            NLUIKeyFrame.set();
        end
    end
    if event == "CHALLENGE_MODE_START" then -- grab the keys level at the start of the key to insert into the table if its completed.
        currentKeyLevel = C_ChallengeMode.GetActiveKeystoneInfo();
        NLUIGVKTCurrentKey = currentKeyLevel;
    end
    if event == "CHALLENGE_MODE_COMPLETED" then -- insert the key level into the list, and sort it, then update the frame after a key to avoid requiring a /reload.
        if firstLogin == true then
            NLUIGVKTList = NLUIGVKTGetWCTList();
            firstLogin = false;
            NLUIKeyFrame.set();
        end
        table.insert(NLUIGVKTList, currentKeyLevel);
        NLUIGVKTList = NLUIGVKTSortKeyList(NLUIGVKTList);
        NLUIKeyFrame.set();
    end
    if event == "CHALLENGE_MODE_KEYSTONE_RECEPTABLE_OPEN" then -- insert the key into the font of power
        NLUIGVKTInsertKey();
    end
    if (event == "CHAT_MSG_PARTY" or "CHAT_MSG_PARTY_LEADER") and NLUIGVKTKeyCommandToggle == true then
        local msg = ...
        if msg == "?keys" then
            local keyLocation = NLUIGKTFindKey();
            SendChatMessage(C_Item.GetItemLink({bagID = keyLocation[1], slotIndex = keyLocation[2]}), "PARTY", nil, nil)
        end
    end
end
NLUIGVKT:SetScript("OnEvent",NLUIGVKTEvents);

-- Title settings
NLUIGVKT.title = NLUIGVKT:CreateFontString(nil, "OVERLAY");
NLUIGVKT.title:SetFontObject("GameFontHighlight");
NLUIGVKT.title:SetPoint("LEFT", NLUIGVKT.TitleBg, "LEFT", 5, -2);
NLUIGVKT.title:SetText("The Great Vault Key Tracker");

-- button for seeing the weekly dungeon reward brackets.
local checkButtons = {};
local function CheckOnClick(self)
  if self:GetChecked() == true then
    for _, button in ipairs(checkButtons) do
        button:SetChecked(false)
    end
    self:SetChecked(true)
  else
  end
  NLUIKeyFrame.set();
end

local endOfWeekRewards = CreateFrame("CheckButton", "MyCheckButton", NLUIGVKT, "ChatConfigCheckButtonTemplate")
endOfWeekRewards:SetPoint("TOPLEFT", NLUIGVKT, "TOPLEFT", 6, -24);
endOfWeekRewards:SetSize(30, 30);
endOfWeekRewards.tooltip = "Weekly Reward Brackets"
table.insert(checkButtons, endOfWeekRewards);
endOfWeekRewards:SetScript("OnClick", CheckOnClick)

local endOfDungeonRewards = CreateFrame("CheckButton", "MyCheckButton", NLUIGVKT, "ChatConfigCheckButtonTemplate")
endOfDungeonRewards:SetPoint("TOPLEFT", NLUIGVKT, "TOPLEFT", 6, -48);
endOfDungeonRewards:SetSize(30, 30);
endOfDungeonRewards.tooltip = "End of Dungeon Rewards"
table.insert(checkButtons, endOfDungeonRewards);
endOfDungeonRewards:SetScript("OnClick", CheckOnClick)

-- toggle the background image
local toggleBackdrop = CreateFrame("CheckButton", "MyCheckButton", NLUIGVKT, "ChatConfigCheckButtonTemplate")
toggleBackdrop:SetPoint("BOTTOMLEFT", NLUIGVKT, "BOTTOMLEFT", 6, 3);
toggleBackdrop:SetSize(30, 30);
toggleBackdrop.tooltip = "Turn Off Background Image"
toggleBackdrop:SetScript("OnClick", function()
    NLUIGVKTBackdropToggle = not NLUIGVKTBackdropToggle
    toggleBackdrop:SetChecked(NLUIGVKTBackdropToggle)
    NLUIKeyFrame.set()
end)

-- outPutFrame is to display NLDestroyList
-- EditBox
NLUIKeyFrame = CreateFrame("Frame", "WeeklyChestTracker", NLUIGVKT)
NLUIKeyFrame:SetSize(275,frameHeight-15); -- width, height.
NLUIKeyFrame.text = NLUIGVKT:CreateFontString(nil, "OVERLAY", "GameTooltipText") 
NLUIKeyFrame.text:SetFont("Fonts\\ARIALN.ttf", 18, "OUTLINE")
NLUIKeyFrame.text:SetPoint("CENTER",0,-9)
NLUIKeyFrame.set = function()-- sets the text to the output frame. use to update it.
    if endOfWeekRewards:GetChecked() == true then
        --local Rewards = NLUIGVKTGetWeeklyRewards();
        NLUIBackdrop:SetTexture()
        NLUIGVKT:SetSize(frameWidth, frameHeight+205)
        NLUIKeyFrame.text:SetText(weeklyIlvlBreakdown);
    elseif endOfDungeonRewards:GetChecked() == true then
        local rewards = NLUIGVKTGetEndOfDungeonRewards();
        NLUIBackdrop:SetTexture()
        NLUIGVKT:SetSize(frameWidth, frameHeight+205)
        NLUIKeyFrame.text:SetText(rewards);
    else
        NLUIGVKT:SetSize(frameWidth+1, frameHeight+1)
        local text = NLUIGVKTTopTenToString(NLUIGVKTList)
        local rewardText = NLUIGVKTGetRewardString(NLUIGVKTList)
        local spec = NLUIGVKTGetLootSpec();
        if NLUIGVKTBackdropToggle == false then
            NLUIBackdrop:SetTexture(backDropTexture, true)
        else
            NLUIBackdrop:SetTexture()
        end
        NLUIKeyFrame.text:SetText(text..rewardText..spec);
    end
end

-- slash commands in game
SLASH_WeeklyChestTracker1 = '/GVKeys';
local function WeeklyChestTracker_handler(msg)
    if msg == "" then
        if firstLogin == true then
            NLUIGVKTList = NLUIGVKTGetWCTList();
            firstLogin = false;
            NLUIKeyFrame.set();
        end
        NLUIGVKT:Show();
    elseif msg == "keylink" then
        if NLUIGVKTKeyCommandToggle == true then
            NLUIGVKTKeyCommandToggle = false;
            print("?keys will no longer link your key");
        else
            NLUIGVKTKeyCommandToggle = true;
            print("?keys will now link your key");
        end
    elseif msg == "renew" then
        NLUIGVKTList = NLUIGVKTGetWCTList();
        firstLogin = false;
        NLUIKeyFrame.set();
        NLUIGVKT:Show();
        print("Great Vault Key Tracker has been reset.")
    elseif msg == "test" then
        NLUIGVKTList = {10,10,10,10,10,10,10,10,10,10}
        NLUIKeyFrame.set();
    end
end
SlashCmdList["WeeklyChestTracker"] = WeeklyChestTracker_handler;